library(ape)
library(picante)
library(geiger)
library(phytools)

#Thanks to Liam Revell for writing this function (http://blog.phytools.org/2012/11/addendum-to-adding-single-tip-to-tree.html)
bind.tip<-function(tree,tip.label,edge.length=NULL, where=NULL,position=0){
  if(is.null(where)) where<-length(tree$tip)+1
  if(is.null(edge.length)&&is.ultrametric(tree)){
    H<-nodeHeights(tree)
    if(where==(length(tree$tip)+1))
      edge.length<-max(H)
    else 
      edge.length<-max(H)-H[tree$edge[,2]==where,2]+position
  }
  tip<-list(edge=matrix(c(2,1),1,2),
            tip.label=tip.label,
            edge.length=edge.length,
            Nnode=1)
  class(tip)<-"phylo"
  obj<-bind.tree(tree,tip,where=where,position=position)
  return(obj)
}

#Thanks to 10k project for providing a template for this function (http://10ktrees.fas.harvard.edu/exercises/10kTrees_Exercise3.pdf)
insertSpecies <- function(original.tree, species.name, sister.species.name, branch.length)
{ 
  speciesToBeInserted = species.name
  insertSpeciesNextTo = sister.species.name
  branchLengthSplitAgo = branch.length
  
  treeNewStr = paste("(", speciesToBeInserted, ":", branchLengthSplitAgo, ");", sep="")
  treeNew <- read.tree(text = treeNewStr)
  # Read tree block
  treeBlock = original.tree
  treeBlockNew = list()
  # For each tree in the tree block
  for (i in 1:length(treeBlock)) { 
    pos = 0
    for (j in 1: length(treeBlock$tip.label)) { if ( insertSpeciesNextTo == treeBlock$tip.label[j]) { pos = j
    }
      
    }
    if (pos == 0) { stop ("Error: Could not find species ",insertSpeciesNextTo, " in tree ",i,", exit\n")
    }
    #Merge both trees and write tree to new list of trees
    treeBlockNew = bind.tree(treeBlock,
                             treeNew,where = pos, position = branchLengthSplitAgo)
  } 
  return(treeBlockNew)
}

#phylogeny from Bininda-Emonds et al. 2007
setwd("/home/alessandro/Documents/writing/papers/artio-roo/")
tree<-read.nexus("./Bininda-emonds_2007_mammals.nex")
tree<-tree$mammalST_bestDates

#get only artiodactyls from our data
all.data<-read.csv("./data-table-macropus-giganteus-averaged.csv",row.names=1)
data <- all.data[which(all.data$order=="Artiodactyla"),]
binomials <- make.names(data$binomial,FALSE)
binomials <- gsub('\\.','_',binomials)

#Adapt Bininda-Emonds et al nomenclature to our nomenclature 
tree$tip.label[which(tree$tip.label=="Elaphurus_davidianus")]="Cervus_davidianus"
tree$tip.label[which(tree$tip.label=="Cephalophus_silvicultor")]="Cephalophus_sylvicultor"

# We hope assuming that Tragulus kanchil and Tragulus napu diverged at roughly the same time as T. javanicus and T. kanchil is accurate enough for our purposes
binomials <- c(binomials, "Tragulus_javanicus")
dummy.data <- as.matrix(rep(1,length(binomials)))
rownames(dummy.data) <- binomials
#replace T. javanicus with T. kanchil (see previous comment)
tree$tip.label[which(tree$tip.label=="Tragulus_javanicus")]="Tragulus_kanchil"

#no other oryx species or close relatives in our data -> we can replace oryx gazella with oryx beisa!
tree$tip.label[which(tree$tip.label=="Oryx_gazella")]="Oryx_beisa"

#insert Bos indicus next to Bos taurus with split 2 mya according to nt sequences from Hiendleder et al. 2008
tree <- insertSpecies(tree, "Bos_indicus", "Bos_taurus", 2)

#correct branch lengths for Cervus davidianus and Cervus elaphus to 4.25 (Pitra et al. 2004)
node.to.shift <- getMRCA(tree, c("Cervus_elaphus","Cervus_davidianus"))
branches.leaving.node.to.shift <- which(tree$edge[,1] %in% node.to.shift)
branches.entering.node.to.shift <- which(tree$edge[,2] %in% node.to.shift)
old.length <- tree$edge.length[branches.leaving.node.to.shift]
new.length <- 4.25
tree$edge.length[branches.leaving.node.to.shift] <- new.length
tree$edge.length[branches.entering.node.to.shift] <- tree$edge.length[branches.entering.node.to.shift]+(old.length-new.length)

#insert Megaloceros giganteus on Cervidae branch, with branch length 5.1 (Pitra et al. 2004), Megaloceros diverged with Dama following Lister et al. 2005
time.until.next.split.on.sister.branch <- 0.85 #=5.1-4.25
branch.length <-5.1
node.to.insert.before <- getMRCA(tree, c("Cervus_davidianus", "Cervus_elaphus"))
tree <- bind.tip(tree,"Megaloceros_giganteus", branch.length, node.to.insert.before, time.until.next.split.on.sister.branch)

missing<-name.check(tree,dummy.data)$tree_not_data
pruned.tree<-drop.tip(tree, missing,root.edge = 1)
pruned.tree<-multi2di(pruned.tree)

png(file = "Artiodactyla-tree.png", width = 600, height = 400)
plot(pruned.tree)
dev.off()

write.tree(pruned.tree, "./Artiodactyla-tree.txt")


