library(ape)
library(picante)
library(geiger)
library(smatr)
library(phytools)
library(nlme)

orders = c("Diprotodontia","Artiodactyla")
all.data<-read.csv("./data-table-macropus-giganteus-averaged.csv",row.names=1)
results <- matrix(nrow=ncol(all.data),ncol=8)

# additional analyses performed for revision
pic.rma.results <- matrix(nrow=ncol(all.data),ncol=6)
pic.ols.results <- matrix(nrow=ncol(all.data),ncol=2)
pgls.rma.results <- matrix(nrow=ncol(all.data),ncol=7)
pgls.ols.results <- matrix(nrow=ncol(all.data),ncol=4)

colnames(results) <- c("slope","lower limit","upper limit","P_value-tested-slope","elevation","r2","P_value-no-correlation", "P_value-no-scaling")

#set up scaling exponents
#isometricExponents = c(rep(1.0/3.0,7),rep(2.0/3.0,7),rep(4.0/3.0,14))
isometricExponents = c(rep(c(1.0/3.0,2.0/3.0,rep(4.0/3.0,2)),7))

for(current.order in orders)
{
  tree <- read.tree(paste0(current.order,"-tree.txt"))
  data <- all.data[which(all.data$order==current.order),]
  mass<-data.frame(data[,"est_body_mass"])
  
  for(current.Y.variable in c(7:34)) {
    test.column<-as.matrix(as.numeric(data[,current.Y.variable]))
    species.names <- make.names(data$binomial, FALSE)
    species.names <- gsub('\\.','_',species.names)
    rownames(mass)<-species.names
    
    test.column<-replace(test.column, is.infinite(log(test.column)),NA)
    regression.data<-cbind(log(mass), log(test.column))
    regression.data<-regression.data[complete.cases(regression.data),]
    regression.data<-as.matrix(regression.data)
    
    missing<-name.check(tree,regression.data)
    pruned.tree<-tree
    
    if(missing[1]!="OK")
    {
      missing<-name.check(tree,regression.data)$tree_not_data
      pruned.tree<-drop.tip(tree, missing,root.edge = 1)
    }
 
    #Phylogenetic independent contrasts for testcol and mass against the tree
    contrasts.data <- apply(as.matrix(regression.data[,2]), 2, pic, pruned.tree)
    contrasts.mass <- apply(as.matrix(regression.data[,1]), 2, pic, pruned.tree)
    
    combined<-cbind(contrasts.mass, contrasts.data)
    contrasts.data<-data.frame(combined)
    
    # PIC
    # option -1 forces fitted line through origin
    contrasts.analysis <- sma(contrasts.data[,2]~contrasts.data[,1]-1,slope.test=isometricExponents[current.Y.variable-6],data=contrasts.data)
    
    results[current.Y.variable,"slope"] <- contrasts.analysis$coef[[1]]["slope","coef(SMA)"]
    results[current.Y.variable,"r2"] <- contrasts.analysis$r2[[1]]
    results[current.Y.variable,"P_value-tested-slope"] <- contrasts.analysis$slopetest[[1]]$p
    results[current.Y.variable,"elevation"] <- contrasts.analysis$coef[[1]]["elevation","coef(SMA)"]
    results[current.Y.variable,"lower limit"] <- contrasts.analysis$coef[[1]]["slope","lower limit"]
    results[current.Y.variable,"upper limit"] <- contrasts.analysis$coef[[1]]["slope","upper limit"]
    results[current.Y.variable,"P_value-no-correlation"] <- contrasts.analysis$pval[[1]]
    
    png(file = paste0(current.order,"/",current.order,"-",colnames(data[current.Y.variable]),"-phy-indep-contrasts.png"), width = 600, height = 400)
    plot(contrasts.data[,2]~contrasts.mass, main=colnames(data[current.Y.variable]))
    dev.off()
    
    # PGLS with phytools
    # this should (and is) the same as pic
    pic.rma.phytools = phyl.RMA(regression.data[,1], regression.data[,2], pruned.tree, "BM", lambda = 1, fixed = TRUE, h0 = isometricExponents[current.Y.variable-6])
    
    colnames(pic.rma.results) <- c("slope","intercept","r2","p-value-tested-slope","lambda","loglikelihood")
    pic.rma.results[current.Y.variable,"slope"] <- pic.rma.phytools$RMA.beta[2]
    pic.rma.results[current.Y.variable,"intercept"] <- pic.rma.phytools$RMA.beta[1]
    pic.rma.results[current.Y.variable,"r2"] <- pic.rma.phytools$test[1]
    pic.rma.results[current.Y.variable,"p-value-tested-slope"] <- pic.rma.phytools$test[4]
    pic.rma.results[current.Y.variable,"lambda"] <- pic.rma.phytools$lambda[1]
    pic.rma.results[current.Y.variable,"loglikelihood"] <- pic.rma.phytools$logL[1]
    
    #this should be the same as PIC, but with OLS, "BM" stands for Brownian motion
    pic.ols.phytools = phyl.resid(tree=pruned.tree, regression.data[,1], regression.data[,2], method = "BM")
    
    colnames(pic.ols.results) <- c("slope","intercept")
    pic.ols.results[current.Y.variable,"slope"] <- pic.ols.phytools$beta[2]
    pic.ols.results[current.Y.variable,"intercept"] <- pic.ols.phytools$beta[1]
    
    #this should optimize the parameter using log likelihood criterion and RMA fitting
    pgls.rma.phytools = phyl.RMA(tree=pruned.tree, regression.data[,1], regression.data[,2], method = "lambda",fixed = FALSE, h0 = isometricExponents[current.Y.variable-6])
    
    colnames(pgls.rma.results) <- c("slope","intercept","r2","p-value-tested-slope","lambda","loglikelihood","h0-slope")
    pgls.rma.results[current.Y.variable,"slope"] <- pgls.rma.phytools$RMA.beta[2]
    pgls.rma.results[current.Y.variable,"intercept"] <- pgls.rma.phytools$RMA.beta[1]
    pgls.rma.results[current.Y.variable,"r2"] <- pgls.rma.phytools$test[1]
    pgls.rma.results[current.Y.variable,"p-value-tested-slope"] <- pgls.rma.phytools$test[4]
    pgls.rma.results[current.Y.variable,"lambda"] <- pgls.rma.phytools$lambda
    pgls.rma.results[current.Y.variable,"loglikelihood"] <- pgls.rma.phytools$logL
    pgls.rma.results[current.Y.variable,"h0-slope"] <- isometricExponents[current.Y.variable-6]
    
    png(file = paste0(current.order,"/",current.order,"-",colnames(data[current.Y.variable]),"-pgls-RMA.png"), width = 600, height = 400)
    plot(pgls.rma.phytools)
    dev.off()
    #this should optimize the parameter using log likelihood criterion and OLS fitting
    pgls.ols.phytools = phyl.resid(tree=pruned.tree, regression.data[,1], regression.data[,2], method = "lambda")
    colnames(pgls.ols.results) <- c("slope","intercept","lambda","loglikelihood")
    pgls.ols.results[current.Y.variable,"slope"] <- pgls.ols.phytools$beta[2]
    pgls.ols.results[current.Y.variable,"intercept"] <- pgls.ols.phytools$beta[1]
    pgls.ols.results[current.Y.variable,"lambda"] <- pgls.ols.phytools$lambda
    pgls.ols.results[current.Y.variable,"loglikelihood"] <- pgls.ols.phytools$logL
    }
  
  rownames(results) = colnames(data)
  rownames(pic.rma.results) = colnames(data)
  rownames(pic.ols.results) = colnames(data)
  rownames(pgls.rma.results) = colnames(data)
  rownames(pgls.ols.results) = colnames(data)
  
  write.table(results, file=paste0(current.order,"/",current.order,"-original-pic.csv"))
  write.table(pic.rma.results, file=paste0(current.order,"/",current.order,"-phytools-rma-pic.csv"))
  write.table(pic.ols.results, file=paste0(current.order,"/",current.order,"-phytools-ols-pic.csv"))
  write.table(pgls.rma.results, file=paste0(current.order,"/",current.order,"-phytools-rma-pgls.csv"))
  write.table(pgls.ols.results, file=paste0(current.order,"/",current.order,"-phytools-ols-pgls.csv"))
}
